@@ -11,6 +11,8 @@
 
 
 #include <stdarg.h>
+#include <string.h>
+#include <fcntl.h>
 #include <string>
 
 #ifdef _OPENMP
@@ -93,7 +95,7 @@ CExpDesign (const string& session_dir_,
                 throw invalid_argument (string ("Failed to canonicalize dir: ") + session_dir_);
 
         if ( session_dir_ != tmp ) {
-                APPLOG_INFO( "canonicalized session_dir \"%s\" to \"%s\"", session_dir_.c_str(), tmp);
+                APPLOG_INFO ("canonicalized session_dir \"%s\" to \"%s\"", session_dir_.c_str(), tmp);
                 _session_dir.assign( tmp);
         } else
                 _session_dir = session_dir_;
@@ -102,11 +104,34 @@ CExpDesign (const string& session_dir_,
         if ( _session_dir.size() > 1 && _session_dir[_session_dir.size()-1] == '/' )
                 _session_dir.erase( _session_dir.size()-1, 1);
 
+        using agh::str::sasprintf;
         if ( fs::exists_and_is_writable( session_dir()) == false )
-                throw invalid_argument (agh::str::sasprintf( "Experiment directory \"%s\" does not exist or is not writable", _session_dir.c_str()));
+                throw invalid_argument (sasprintf( "Experiment directory\n"
+                                                   "<i>%s</i>\n"
+                                                   "does not exist or is not writable", session_dir()));
 
         if ( chdir( session_dir()) == -1 )
-                throw invalid_argument (agh::str::sasprintf( "Failed to cd to \"%s\"", _session_dir.c_str()));
+                throw invalid_argument (sasprintf( "Failed to cd to \"%s\"", session_dir()));
+
+        {
+                string lock_fname (_session_dir + "/.session.lock");
+                struct stat stat0;
+                int stst = stat( lock_fname.c_str(), &stat0);
+                if ( stst == 0 ) {
+                        const char *lock_mtime_s = ctime( &stat0.st_mtime);
+                        throw invalid_argument
+                                (sasprintf("A lock file exists:\n<i>%s</i>,\ncreated %.*s).\n\n"
+                                           "If this lock is stale, delete it and try again.",
+                                           lock_fname.c_str(), (int)strlen(lock_mtime_s)-1, lock_mtime_s));
+                }
+                open( lock_fname.c_str(), O_WRONLY|O_CREAT|O_TRUNC, 0600);
+                stst = stat( lock_fname.c_str(), &stat0);
+                if ( stst != 0 )
+                        throw invalid_argument
+                                (sasprintf("Failed to place a session lock file (%s)",
+                                           strerror(errno)));
+        }
+
 
         load_settings();
 
@@ -131,6 +156,13 @@ CExpDesign (const string& session_dir_,
 }
 
 
+CExpDesign::~CExpDesign ()
+{
+        save_settings();
+        unlink((_session_dir + "/.session.lock").c_str());
+}
+
+
 void
 agh::CExpDesign::
 log_message( TLogEntryStyle style, const char* fmt, ...)
